import os
import sys
from .__config__ import _ask, QproConfig, QproDefaultConsole

if sys.platform.startswith('win'):
    is_win = True
    dir_char = '\\'
else:
    is_win = False
    dir_char = '/'

user_root = os.path.expanduser('~')
_qpro_config = QproConfig(user_root + dir_char + '.qprorc', os.path.exists(user_root + dir_char + '.qprorc'))

try:
    import QuickStart_Rhy
    qs_flag = True
except:
    qs_flag = False

user_lang = _qpro_config.select('default_language')
user_pip = _qpro_config.select('default_pip')
QproErrorString = '[bold red][ERRO]' if user_lang != 'zh' else '[bold red][错误]'
QproInfoString = '[bold cyan][INFO]' if user_lang != 'zh' else '[bold cyan][提示]'
QproWarnString = '[bold yellow][WARN]' if user_lang != 'zh' else '[bold yellow][警告]'
name = 'QuickProject'
configure_name = 'project_configure.json'


def __latest_filename(filename):
    """
    获取最近的文件名 （不断向父目录遍历）

    :param filename: 文件名
    :return: 文件路径
    """
    import os

    cur = os.getcwd()
    rec = cur
    while cur != dir_char:
        if os.path.exists(filename):
            os.chdir(rec)
            return os.path.abspath(cur + dir_char + filename)
        os.chdir('..')
        last = cur
        cur = os.getcwd()
        if cur == last:
            break
    os.chdir(rec)
    return ''


if ' '.join(sys.argv[:2]) != 'Qpro -init':
    rt_dir = os.path.dirname(__latest_filename(configure_name)) + dir_char
    if rt_dir == dir_char:
        rt_dir = os.path.abspath('.') + dir_char
else:
    rt_dir = os.path.abspath('.')
project_configure_path = rt_dir + configure_name


def __sub_path(path, isExist=True):
    """
    计算路径在项目中的相对路径

    :param path: 路径
    :param isExist: 是否存在
    :return: 相对路径
    """
    if not os.path.exists(path) and isExist:
        return ''
    abs_path = os.path.abspath(path)
    return abs_path.replace(rt_dir, '') if abs_path.startswith(rt_dir) else ''


def requirePackage(pname: str, module: str = "", real_name: str = "", not_exit: bool = True, not_ask: bool = False, set_pip: str = user_pip):
    """
    获取本机上的python第三方库

    :param pname: 库名
    :param module: 待引入的模块名，可缺省
    :param real_name: 用于 pip3 install 的名字
    :param not_exit: 安装后不退出
    :return: 库或模块的地址
    """
    try:
        exec(f'from {pname} import {module}' if module else f"import {pname}")
    except (ModuleNotFoundError, ImportError):
        if not_ask:
            return None
        confirm = _ask({
            'type': 'confirm',
            'name': 'install',
            'message': f"""Qs require {pname + (' -> ' + module if module else '')}, confirm to install?  
  Qs 依赖 {pname + (' -> ' + module if module else '')}, 是否确认安装?""",
            'default': True})
        if confirm:
            os.system(f'{set_pip} install {pname if not real_name else real_name} -U')
            if not_exit:
                exec(f'from {pname} import {module}' if module else f"import {pname}")
            else:
                QproDefaultConsole.print(QproInfoString, f'just run again: "{" ".join(sys.argv)}"' if user_lang != 'zh' else f'请重新运行: "{" ".join(sys.argv)}"')
                exit(0)
        else:
            exit(-1)
    finally:
        return eval(f'{module if module else pname}')


class SshProtocol:
    @staticmethod
    def post_folder(user, domain, target, port, srcPath, dstPath):
        """
        发送目录

        scp -P {port} -r {srcPath} {user}@{domain}:{target}/{dstPath}

        :param user: 用户名，可省略
        :param domain: 目标机器地址
        :param target: 远程映射路径
        :param port: 端口
        :param srcPath: 本地文件路径
        :param dstPath: 远程映射子路径
        :return: 发送状态
        """
        if user:
            status = os.system(
                'scp -P %s -r %s %s' % (port, srcPath, user + '@\\[' + domain + '\\]:' + target + dstPath))
        else:
            status = os.system('scp -P %s -r %s %s' % (port, srcPath, '\\[' + domain + '\\]:' + target + dstPath))
        return status

    @staticmethod
    def post_file(user, domain, target, port, srcPath, dstPath):
        """
        发送文件

        scp -P {port} {srcPath} {user}@{domain}:{target}/{dstPath}

        :param user: 用户名，可省略
        :param domain: 目标机器地址
        :param target: 远程映射路径
        :param port: 端口
        :param srcPath: 本地文件路径
        :param dstPath: 远程映射子路径
        :return: 发送状态
        """
        if user:
            status = os.system('scp -P %s %s %s' % (port, srcPath, user + '@\\[' + domain + '\\]:' + target + dstPath))
        else:
            status = os.system('scp -P %s %s %s' % (port, srcPath, '\\[' + domain + '\\]:' + target + dstPath))
        return status

    @staticmethod
    def post_all_in_folder(user, domain, target, port, dstPath):
        """
        发送当前目录

        scp -P {port} -r * {user}@{domain}:{target}/{dstPath}

        :param user: 用户名，可省略
        :param domain: 目标机器地址
        :param target: 远程映射路径
        :param port: 端口
        :param srcPath: 本地文件路径
        :param dstPath: 远程映射子路径
        :return: 发送状态
        """
        if user:
            status = os.system('scp -P %s -r * %s' % (port, user + '@\\[' + domain + '\\]:' + target + dstPath))
        else:
            status = os.system('scp -P %s -r * %s' % (port, '\\[' + domain + '\\]:' + target + dstPath))
        return status

    @staticmethod
    def get_file_or_folder(user, domain, target, port, srcPath, dstPath):
        """
        下载文件或目录

        scp -P {port} -r {user}@{domain}:{target}/{srcPath} {dstPath}

        :param user: 用户名，可省略
        :param domain: 目标机器地址
        :param target: 远程映射路径
        :param port: 端口
        :param srcPath: 远程映射子路径
        :param dstPath: 本地文件路径
        :return: 发送状态
        """
        if user:
            return os.system('scp -P %s -r %s %s' % (port, user + '@\\[' + domain + '\\]:' + target + srcPath, dstPath))
        else:
            return os.system('scp -P %s -r %s %s' % (port, '\\[' + domain + '\\]:' + target + srcPath, dstPath))

    @staticmethod
    def command(user, domain, target, port, command):
        """
        在远程映射执行命令

        :param user: 用户名，可省略
        :param domain: 目标机器地址
        :param target: 远程映射路径
        :param port: 端口
        :param command: 命令
        :return: 命令输出结果
        """
        if not domain or not target:
            return 0
        import subprocess
        cmd = ['ssh', '-p', port, user + '@' + domain if user else domain, f'cd {target}', ';', command]
        return subprocess.check_output(cmd).decode('utf-8')

    @staticmethod
    def ssh(user, domain, target, port, dstPath):
        """
        连接至远程映射控制台

        :param user: 用户名，可省略
        :param domain: 目标机器地址
        :param target: 远程映射路径
        :param port: 端口
        :param dstPath: 子路径
        :return:
        """
        if user:
            return os.system(
                "ssh -p {port} -t {user}@{domain} 'cd {aim} ; exec $SHELL -l'".format(
                    port=port, user=user, domain=domain, aim=target + dstPath
                )
            )
        else:
            return os.system(
                "ssh -p {port} -t {domain} 'cd {aim} ; exec $SHELL -l'".format(
                    port=port, domain=domain, aim=target + dstPath
                )
            )


def menu_output(menu):
    from rich.table import Table, Column
    from rich.box import SIMPLE

    tb = Table(*[Column('Parameter', justify='full', style='bold yellow'),
                 Column('Description', justify='right', style='bold cyan')],
               show_edge=False, show_header=False, row_styles=['none', 'dim'], box=SIMPLE, pad_edge=False,
               title='[bold underline] {title}[dim]Author: RhythmLian\n'.format(title=menu['title']))
    for line in menu['lines']:
        tb.add_row((menu['prefix'] + ' ' if line[0].startswith('-') else '') + line[0], line[1])
    QproDefaultConsole.print(tb, justify='center')
    QproDefaultConsole.print(
        '\nDOC:' if user_lang != 'zh' else '\n文档:', 'https://rhythmlian.cn/2020/02/14/QuickProject/', justify='center'
    )


def get_config(without_output: bool = False):
    config_path = __latest_filename('project_configure.json')
    config = {}
    if config_path:
        import json
        with open(config_path, 'r') as f:
            config = json.load(f)
    else:
        if not without_output:
            QproDefaultConsole.print(
                QproErrorString,
                "No file named: project_configure.json\n May you need run:\"Qpro -init\" first!"
                if user_lang != 'zh' else
                "没有文件: project_configure.json\n可能你需要先运行: \"Qpro -init\"!"
            )
        exit(0)
    return config


def get_server_targets():
    res = get_config()['server_targets']
    for item in res:
        if not item['path'].endswith('/'):
            item['path'] += '/'
    return res


def _choose_server_target():
    server_targets = get_server_targets()
    choices = [
        f'{i["user"]}@{i["host"]}:{i["path"]} port: {i["port"]}'
        if i['user'] else
        f'{i["host"]}:{i["path"]} port: {i["port"]}' for i in server_targets if i['host'] and i['path']]
    try:
        index = choices.index(_ask({
            'type': 'list',
            'name': 'index',
            'message': 'Choose target | 选择目标:',
            'choices': choices
        }))
        return server_targets[index]
    except:
        return None



from . import *
from . import _ask
from QuickProject import __sub_path, _choose_server_target
from rich.prompt import Prompt


def __format_json(info, path: str):
    """
    回写配置表

    :param info: 列表格式或字典格式
    :param path: 路径
    :return:
    """
    import json
    with open(path, 'w') as f:
        if isinstance(info, list):
            config = {}
            for line in info:
                config[line[0]] = line[1] if line[0] != 'server_targets' else line[1:]
            json.dump(config, f, indent=1)
        elif isinstance(info, dict):
            json.dump(info, f, indent=1)


def __findAndReplace(dirPath, fo, to):
    """
    替换模板项目中的指定字段

    :param dirPath: 项目路径
    :param fo: 识别
    :param to: 替换
    :return:
    """
    for rt, son_dir, son_files in os.walk(dirPath):
        for file in son_files:
            with open(rt + dir_char + file, 'r') as f:
                ct = f.read()
            with open(rt + dir_char + file, 'w') as f:
                f.write(ct.replace(fo, to))


def remove(path):
    """
    删除文件或目录

    :param path: 路径
    :return:
    """
    import shutil
    if os.path.exists(path):
        if os.path.isdir(path):
            shutil.rmtree(path)
        else:
            os.remove(path)


def scp_init(server_targets: list):
    if not server_targets:
        return
    for server_target in server_targets:
        user, host, port, target = server_target['user'], server_target['host'], server_target['port'], server_target['path']
        st = SshProtocol.post_all_in_folder(user, host, target, port, '')
        if st:
            QproDefaultConsole.print(QproErrorString, "upload project failed!" if user_lang != 'zh' else '上传项目失败!')


def _create_empty_project(project_name):
    if os.path.exists(project_name):
        return QproDefaultConsole.print(
            QproErrorString,
            f'{project_name} already exists!' if user_lang != 'zh' else f'{project_name} 已存在'
        )
    os.mkdir(project_name)
    __format_json([
        ['compile_tool', ''],
        ['compile_filename', ''],
        ['executable_filename', ''],
        ['input_file', ''],
        ['template_root', ''],
        ['server_target', '', '']
    ], project_name + dir_char + configure_name)
    os.chdir(project_name)
    return


def _search_supported_languages(is_CN):
    kw = _ask({
        'type': 'input',
        'message': 'Input a keyword' if user_lang != 'zh' else '输入一个关键词',
        'name': 'language'
    })

    import json
    import requests

    try:
        res = json.loads(requests.get(f'https://qpro.rhythmlian.cn/?keyword={kw}&is_CN={str(is_CN).lower()}').text)
        if not res['status']:
            QproDefaultConsole.print(QproErrorString, res['message'])
            return None
        data = {i[0]: i[1:] for i in res['data']}
        return data[
            _ask({
                'type': 'list',
                'message': 'Choose Supported Language' if user_lang != 'zh' else '选择支持的语言',
                'name': 'url',
                'choices': list(data.keys())
            })
        ]
    except Exception as e:
        QproDefaultConsole.print(QproErrorString, repr(e))
    return None


def _external_create(project_name: str, key: str = ''):
    from git import Repo

    if key:
        if key in ['Empty', '空白项目']:
            return _create_empty_project(project_name)

        try:
            from QuickStart_Rhy.API.alapi import ip_info
            with QproDefaultConsole.status(
                    ('Check IP to switch mirrors' if user_lang != 'zh' else '检查IP以选择合适的镜像').format(key, project_name)):
                is_CN = ip_info('')['ad_info']['nation'].startswith('中国')
        except:
            is_CN = False

        templateProjectUrls = _search_supported_languages(is_CN)
        if not templateProjectUrls:
            exit(0)
        with QproDefaultConsole.status(
                ('Cloning Qpro {} Template to {}' if user_lang != 'zh' else '正在克隆Qpro {} 模板为 {}').format(key,
                                                                                                         project_name)):
            Repo.clone_from(templateProjectUrls[0], project_name)
    else:
        templateProjectUrls = _ask({
            'type': 'input',
            'message': 'GIT ' + ('URL' if user_lang != 'zh' else '链接') + ':',
            'name': 'url'
        })
        with QproDefaultConsole.status(
                ('Cloning External Template to {}' if user_lang != 'zh' else '正在克隆Qpro 外部模板为 {}').format(project_name)):
            Repo.clone_from(templateProjectUrls, project_name)
    os.chdir(project_name)
    try:
        remove('.git')
    except Exception as e:
        QproDefaultConsole.print(QproErrorString, repr(e))
    if key:
        __findAndReplace(os.getcwd(), templateProjectUrls[1], project_name)


def __get_server_target_from_string():
    """
    从字符串中获取服务器目标
    """
    server_target = _ask({
        'type': 'input',
        'name': 'server_target',
        'message': 'Input user@ip:dir_path if you need scp' if user_lang != 'zh' else '输入 用户@IP:路径 如果你打算使用SSH'
    }).strip().replace(dir_char, '/')

    if server_target and not server_target.endswith('/'):
        if not server_target.endswith(':'):
            server_target += '/'
        else:
            server_target += '~/'
    user, host, path, port = '', '', '', 22
    if server_target:
        user, target = server_target.split('@')
        target = target.split(':')
        host = ':'.join(target[:-1])
        path = target[-1] + ('' if target[-1].endswith('/') else '/')
        port = _ask({
            'type': 'input',
            'name': 'port',
            'message': 'input port' if user_lang != 'zh' else '输入端口号',
            'default': '22'
        })
    return {
        'user': user,
        'host': host,
        'port': port,
        'path': path
    }


def create():
    try:
        project_name = sys.argv[2]
    except IndexError:
        return QproDefaultConsole.print(QproWarnString,
                                        'usage: Qpro create <project>' if user_lang != 'zh' else '使用: Qpro create <项目>')
    else:
        if os.path.exists(project_name) and os.path.isdir(project_name):
            return QproDefaultConsole.print(QproErrorString, '"%s" is exist!' % (os.path.abspath(project_name)))

        lang = _ask({
            'type': 'list',
            'name': 'lang_name',
            'message': 'Choose Lang | 选择语言:',
            'choices': [
                'Empty' if user_lang != 'zh' else '空白项目',
                'Template' if user_lang != 'zh' else '内置模板',
                'External' if user_lang != 'zh' else '外部项目'
            ]
        })

        if lang in ['external', '外部项目']:
            _external_create(project_name)
        else:
            _external_create(project_name, lang)

        config = get_config()
        config['server_targets'] = [__get_server_target_from_string()]
        __format_json(config, configure_name)

        if _ask({
            'type': 'confirm',
            'message': 'Open it with VS Code?' if user_lang != 'zh' else '是否现在使用VS Code打开?',
            'name': 'open',
            'default': True
        }):
            if sys.platform == 'darwin':
                os.system(f'open -a "Visual Studio Code" .')
            else:
                os.system('code .')


def scp(smv_flag: bool = False):
    kw = 'scp' if not smv_flag else 'smv'
    try:
        path = sys.argv[2]
        sub_path = __sub_path(path)
    except IndexError:
        return QproDefaultConsole.print(
            QproWarnString, f'usage: Qpro {kw} <path>' if user_lang != 'zh' else f'使用: Qpro {kw} <路径>'
        )
    else:
        if not sub_path:
            return \
                QproDefaultConsole.print(
                    QproErrorString,
                    (f"{path} is not in this Qpro project!" if os.path.exists(path) else f'No such file named: {path}')
                    if user_lang != 'zh' else
                    (f"{path} 不在当前 Qpro 项目中!" if os.path.exists(path) else f'该路径不存在: {path}')
                )
        server_targets = get_server_targets()
        status = 0
        for server_target in server_targets:
            if os.path.isdir(path):
                status |= SshProtocol.post_folder(server_target['user'], server_target['host'], server_target['path'], server_target['port'], path, sub_path)
            else:
                status |= SshProtocol.post_file(server_target['user'], server_target['host'], server_target['path'], server_target['port'], path, sub_path)
        if smv_flag and not status:
            remove(path)


def smv():
    if _ask({
        'type': 'confirm',
        'message': 'Confirm delete after transform | 确认传输后删除',
        'default': True,
        'name': 'confirm'
    }):
        scp(True)


def get():
    try:
        path = sys.argv[2]
        sub_path = __sub_path(path, isExist=False)
    except IndexError:
        return QproDefaultConsole.print(
            QproWarnString, 'usage: Qpro get <path>' if user_lang != 'zh' else '使用: Qpro get <路径>'
        )
    else:
        if not sub_path:
            return QproDefaultConsole.print(QproErrorString,
                                            f"{path} is not in this Qpro project!" if user_lang != 'zh' else f'{path} 不在当前 Qpro 项目中!')
        server_target = get_server_targets()[0]
        SshProtocol.get_file_or_folder(server_target['user'], server_target['host'], server_target['port'], server_target['path'], sub_path, path)


def pro_init():
    work_project = ''
    while not work_project:
        work_project = Prompt.ask('You need to set project name' if user_lang != 'zh' else '请设置项目名').strip()
    lang_tool_exe = {
        'c': ['gcc -std=c11 --source_file-- -o --execute--', '', '.c'],
        'cpp': ['g++ -std=c++11 --source_file-- -o --execute--', '', '.cpp'],
        'java': ['javac -d dist --source_file--', 'java -classpath dist ', '.java'],
        'python3': ['', 'python3 ', '.py'],
        'python': ['', 'python ', '.py'],
        'empty | other': ['', '', '']
    }

    lang_name = _ask({
        'type': 'list',
        'name': 'lang_name',
        'message': 'Choose Lang | 选择语言:',
        'choices': lang_tool_exe.keys()
    })

    lang = lang_tool_exe[lang_name]
    source_file = ''
    if lang_name != 'empty | other':
        source_file = ('main' + lang[-1]) if lang[0] != 'javac' else work_project + lang[-1]
        while not os.path.exists(source_file):
            source_file = Prompt.ask((
                                         'Not found "%s", set compile_filename'
                                         if user_lang != 'zh' else
                                         '没有找到 "%s", 请设置源文件'
                                     ) % source_file).strip()
    if lang[0] != 'javac':
        execute = lang[1] + 'dist' + dir_char + work_project if lang[0] else lang[1] + source_file
    elif lang_name != 'empty | other':
        work_project = source_file.split(dir_char)[-1].split('.')[0]
        execute = lang[1] + work_project
    else:
        execute = ''
    if (not os.path.exists('dist') or not os.path.isdir('dist')) and lang_name != 'empty | other':
        os.mkdir('dist')
    info = [
        ['compile_tool', lang[0].replace('--source_file--', source_file).replace('--execute--', execute)],
        ['compile_filename', source_file],
        ['executable_filename', execute],
        ['input_file', 'dist' + dir_char + 'input.txt' if lang_name != 'empty' else ''],
        ['template_root', 'template' + dir_char if lang_name != 'empty' else ''],
        ['server_targets', __get_server_target_from_string()],
    ]
    __format_json(info, configure_name)
    if lang_name and lang_name == 'empty | other':
        if _ask({
            'type': 'confirm',
            'message': 'Confirm to sync project | 确认同步项目',
            'default': True,
            'name': 'confirm'
        }):
            scp_init(info[-1][1:])
        return
    with open(info[3][-1], 'w') as f:
        f.write('edit this file to make input' if user_lang != 'zh' else '编辑此文件作为程序输入')
    if not os.path.exists('template') or not os.path.isdir('template'):
        os.mkdir('template')
    try:
        with open(info[1][-1], 'r') as f:
            main_cont = f.read()
        with open('template' + dir_char + 'main', 'w') as f:
            f.write(main_cont)
    except Exception as e:
        QproDefaultConsole.print(
            QproErrorString, "make backup failed with error: %s, you need backup code by yourself!" % e)
    if _ask({
        'type': 'confirm',
        'message': 'Confirm to sync project | 确认同步项目',
        'default': True,
        'name': 'confirm'
    }):
        scp_init(info[-1][1:])


def ssh():
    server_target = _choose_server_target()
    if not server_target:
        return
    SshProtocol.ssh(server_target['user'], server_target['host'], server_target['path'], server_target['port'], __sub_path('.'))


def delete_all():
    server_targets = get_server_targets()
    st = 0
    for server_target in server_targets:
        _st = SshProtocol.command(server_target['user'], server_target['host'], server_target['path'], server_target['port'], 'rm -rf .')
        st |= 1 if _st else 0
        QproDefaultConsole.print(QproErrorString, f'{server_target}: delete all failed with error: {st}')
    if not st:
        remove(os.getcwd())


def delete():
    try:
        path = os.path.abspath(sys.argv[2])
        sub_path = __sub_path(path)
    except IndexError:
        return QproDefaultConsole.print(
            QproWarnString, 'usage: Qpro del <path>' if user_lang != 'zh' else '使用: Qpro del <路径>'
        )
    else:
        if not sub_path:
            return \
                QproDefaultConsole.print(
                    QproErrorString,
                    (f"{path} is not in this Qpro project!" if os.path.exists(path) else f'No such file named: {path}')
                    if user_lang != 'zh' else
                    (f"{path} 不在当前 Qpro 项目中!" if os.path.exists(path) else f'该路径不存在: {path}')
                )
        server_targets = get_server_targets()
        st = 0
        for server_target in server_targets:
            _st = SshProtocol.command(server_target['user'], server_target['host'], server_target['path'],
                                      server_target['port'], f'rm -rf {sub_path}')
            st |= 1 if _st else 0
            if _st:
                QproDefaultConsole.print(QproErrorString, f'{server_target}: delete {sub_path} failed with error: {_st}')
        if not st or _ask({
            'type': 'confirm',
            'name': 'confirm',
            'message': f'{path} is not in this Qpro project! Do you want to delete it?' if user_lang != 'zh' else f'{path} 不在当前 Qpro 项目中! 是否删除?',
        }):
            remove(sub_path)


def tele_ls():
    try:
        path = sys.argv[2]
        sub_path = __sub_path(path, False)
    except IndexError:
        sub_path = __sub_path('./', False)
    server_target = _choose_server_target()
    if server_target:
        from . import SshProtocol
        res = SshProtocol.command(server_target['user'], server_target['host'], server_target['path'], server_target['port'], f'ls -lah {sub_path}').strip().split('\n')[3:]
        res = [i.strip().split() for i in res]

        from rich.table import Table
        from rich.box import SIMPLE_HEAVY
        from rich.color import Color
        from rich.text import Text
        from rich.style import Style

        table = Table(title=f'{sub_path}', box=SIMPLE_HEAVY, show_header=True, header_style='bold magenta')
        table.add_column('permission' if user_lang != 'zh' else '权限码', justify='center')
        table.add_column('size' if user_lang != 'zh' else '尺寸', justify='center')
        table.add_column('owner' if user_lang != 'zh' else '拥有者', justify='center')
        table.add_column('time' if user_lang != 'zh' else '修改时间', justify='center')
        table.add_column('name' if user_lang != 'zh' else '文件名', justify='right')
        for i in res:
            is_dir = True if i[0][0] == 'd' else False
            color = Color.from_rgb(112, 87, 250) if is_dir else None
            p_color = Color.from_rgb(171, 157, 242)
            permission = ''.join(['1' if j != '-' else '0' for j in i[0][1:]])
            permission = '%d%d%d' % (int(permission[:3], 2), int(permission[3: 6], 2), int(permission[6:], 2))
            table.add_row(*[Text(permission, style=Style(color=p_color)), f'[green]{i[4]}', '[bold yellow]' + i[2], '[blue]' +  ' '.join(i[5:8]), Text(" ".join(i[8:]), style=Style(color=color, bold=is_dir))])
        QproDefaultConsole.print(table, justify='center')


def template_format():
    config = {}
    with open('project_configure.csv', 'r') as f:
        for row in f.read().strip().split('\n'):
            row = row.replace('\,', '--QPRO-IS-SPLIT--')
            row = [i.replace('--QPRO-IS-SPLIT--', ',') for i in row.split(',')]
            config[row[0]] = [i.strip() for i in row[1:]]
        for i in config:
            if i in ['server_target']:
                continue
            config[i] = config[i][0]
    __format_json(config, 'project_configure.json')
    remove('project_configure.csv')


def enable_complete():
    config = get_config()
    config['enable_complete'] = True
    __format_json(config, project_configure_path)


def __get_Qpro_fig_Dir():
    QproGlobalDir = os.environ.get('QproGlobalDir', None)
    if not QproGlobalDir:
        QproDefaultConsole.print(QproErrorString,
                                 'QproGlobalDir is not set!' if user_lang != 'zh' else 'QproGlobalDir 未设置!')
        return None
    QproGlobalDir = QproGlobalDir.replace('~', user_root)
    if not os.path.exists(QproGlobalDir):
        QproDefaultConsole.print(QproErrorString,
                                 'QproGlobalDir is not exists!' if user_lang != 'zh' else 'QproGlobalDir 不存在!')
        return None
    if not os.path.exists(os.path.join(QproGlobalDir, 'bin')):
        os.mkdir(os.path.join(QproGlobalDir, 'bin'))
    if not os.path.exists(os.path.join(QproGlobalDir, 'fig')):
        os.mkdir(os.path.join(QproGlobalDir, 'fig'))
    return QproGlobalDir


def register_global_command():
    if not os.path.exists(configure_name):
        return QproDefaultConsole.print(QproErrorString, f'{configure_name} is not exists!' if user_lang != 'zh' else f'{configure_name} 不存在!')
    if is_win:
        return QproDefaultConsole.print(QproWarnString, 'Not Support Windows!' if user_lang != 'zh' else '不支持 Windows!')
    QproGlobalDir = __get_Qpro_fig_Dir()
    if not QproGlobalDir:
        return
    import json

    disable_global_command = '--disable_global_command' in sys.argv
    project_name = os.getcwd().split(dir_char)[-1]
    fig_dir = os.path.join(QproGlobalDir, 'fig')
    if os.path.exists(os.path.join(fig_dir, f'{project_name}.json')):
        QproDefaultConsole.print(QproWarnString,
                                 'This project has been registered!' if user_lang != 'zh' else '该项目已注册!')
        if _ask({
            'type': 'confirm',
            'name': 'confirm',
            'message': 'Do you want to override it?' if user_lang != 'zh' else '是否覆盖?',
            'default': False
        }):
            os.remove(os.path.join(fig_dir, f'{project_name}.json'))
        else:
            return
    import subprocess

    with open(os.path.join(fig_dir, f'{project_name}.json'), 'w') as f:
        project_subcommands = json.loads(subprocess.check_output(['qrun', '--qrun-fig-complete']).decode('utf-8'))
        if not project_subcommands:
            return QproDefaultConsole.print(QproErrorString, 'Not a Commander APP' if user_lang != 'zh' else '不是Commander应用')
        for item in project_subcommands:
            if 'args' in item:
                if 'options' not in item:
                    item['options'] = []
                for arg in item['args']:
                    if arg['name'].startswith('-') and not arg['name'].startswith('--'):
                        if 'file' in arg['name'] or 'path' in arg['name']:
                            arg['args']['template'] = ['filepaths', 'folders']
                        item['options'].append(arg)
                        item['args'].remove(arg)
                        continue
                    if 'file' in arg['name'] or 'path' in arg['name']:
                        arg['template'] = ['filepaths', 'folders']

        json.dump({
            'fig': {
                'name': project_name,
                'description': project_name,
                'subcommands': project_subcommands
            },
            'path': os.getcwd()
        }, f, ensure_ascii=False, indent=1)
    if disable_global_command:
        return
    with open(os.path.join(QproGlobalDir, 'bin', f'{project_name}'), 'w') as f:
        with open(get_config()['compile_filename'], 'r') as ff:
            ct = ff.read()
        if not ct.startswith('#!/usr/bin/env python3'):
            ct = '#!/usr/bin/env python3\n' + ct
        f.write(ct)
    os.chmod(os.path.join(QproGlobalDir, 'bin', f'{project_name}'), 0o755)


def gen_fig_script():
    if is_win:
        return QproDefaultConsole.print(QproWarnString, 'Not Support Windows!' if user_lang != 'zh' else '不支持 Windows!')
    QproGlobalDir = __get_Qpro_fig_Dir()
    if not QproGlobalDir:
        return
    import json
    from .QproFigTable import default_custom_command_template
    fig_Qpro_template = requirePackage('.QproFigTable', f'default_Qpro_{user_lang if user_lang == "zh" else "en"}_content_template')
    fig_qrun_template = requirePackage('.QproFigTable', f'default_qrun_{user_lang if user_lang == "zh" else "en"}_content_template')
    fig_detector_template = requirePackage('.QproFigTable', f'default_detector_{user_lang if user_lang == "zh" else "en"}_content_template')
    fig_tmpm_template = requirePackage('.QproFigTable', f'default_tmpm_{user_lang if user_lang == "zh" else "en"}_content_template')

    with open('src/Qpro.ts', 'w') as f:
        subcommands = os.listdir(os.path.join(QproGlobalDir, 'fig'))
        res = []
        for cmd in subcommands:
            with open(os.path.join(QproGlobalDir, 'fig', cmd), 'r') as cmdf:
                proj_name = cmd.split(".")[0]
                fig = json.load(cmdf)['fig']
                res.append(fig)
                if not os.path.exists(os.path.join(QproGlobalDir, 'bin', f'{proj_name}')):
                    continue
                with open(f'src/{proj_name}.ts', 'w') as custom_f:
                    custom_f.write(default_custom_command_template.replace('__CUSTOM_COMMAND_SPEC__', json.dumps(fig, ensure_ascii=False, indent=1)))
        f.write(fig_Qpro_template.replace('--Qpro-bin-subcommands-gen--', json.dumps(res, ensure_ascii=False, indent=1)))
    with open('src/qrun.ts', 'w') as f:
        f.write(fig_qrun_template)
    with open('src/detector.ts', 'w') as f:
        f.write(fig_detector_template)
    with open('src/tmpm.ts', 'w') as f:
        f.write(fig_tmpm_template)
    os.system('npm run copy-to-global')


def bin_exec():
    if is_win:
        return QproDefaultConsole.print(QproWarnString, 'Not Support Windows!' if user_lang != 'zh' else '不支持 Windows!')
    import json
    proj_name = sys.argv[2]
    proj_args = sys.argv[3:]
    QproGlobalDir = __get_Qpro_fig_Dir()
    if not QproGlobalDir:
        return
    if f'{proj_name}.json' not in os.listdir(os.path.join(QproGlobalDir, 'fig')):
        return QproDefaultConsole.print(QproErrorString, f'{proj_name} is not registered' if user_lang == 'zh' else f'{proj_name} 未注册')
    with open(os.path.join(QproGlobalDir, 'fig', f'{proj_name}.json'), 'r') as f:
        path = json.load(f)['path']
    os.chdir(path)
    os.system(f'qrun ' + ' '.join(proj_args))


def gen_zsh_comp():
    if is_win:
        return QproDefaultConsole.print(QproWarnString, 'Not Support Windows!' if user_lang != 'zh' else '不支持 Windows!')
    QproGlobalDir = __get_Qpro_fig_Dir()
    if not QproGlobalDir:
        return

    import json
    from .QproZshComp import zsh_comp_template

    subcommands = os.listdir(os.path.join(QproGlobalDir, 'fig'))
    file_comp1 = """
    else
        _arguments -S -s '*:filename:_files'
        return
    fi
    """
    file_comp2 = """
    _arguments -S -s '*:filename:_files'
    return
    """
    for cmd in subcommands:
        proj_name = cmd.split(".")[0]
        if not os.path.exists(os.path.join(QproGlobalDir, 'bin', f'{proj_name}')):
            continue
        with open(os.path.join(QproGlobalDir, 'fig', cmd), 'r') as cmdf:
            fig = json.load(cmdf)['fig']
        cur_sub_cmds = []
        sub_cmd_args = []
        for sub_cmd in fig['subcommands']:
            cur_sub_cmds.append(f"{sub_cmd['name']}:'{sub_cmd['description']}'")
            if 'args' in sub_cmd and sub_cmd['args']:
                cur_args = """if [[ ${prev} == __sub_cmd__ ]]; then
        opt_args=(
            __sub_cmd_opts__
        )"""
                sub_cmd_opts = []
                for arg in sub_cmd['args']:
                    if arg['name'].startswith('-'):
                        sub_cmd_opts.append(f"{arg['name']}:'{arg['description']}'")
                for opt in sub_cmd['options']:
                    if opt['name'].startswith('-'):
                        sub_cmd_opts.append(f"{opt['name']}:'{opt['description']}'")
                cur_args = cur_args.replace('__sub_cmd__', sub_cmd['name'])
                cur_args = cur_args.replace('__sub_cmd_opts__', '\n            '.join(sub_cmd_opts))
                sub_cmd_args.append(cur_args)

        with open('_' + fig['name'], 'w') as f:
            template = zsh_comp_template
            template = template.replace('__proj_name__', fig['name'])
            template = template.replace('__sub_commands__', '\n        '.join(cur_sub_cmds))
            template = template.replace('__sub_commands_args__', '\n    el'.join(sub_cmd_args) + file_comp1 if sub_cmd_args else file_comp2)
            f.write(template)


func = {
    'create': create,
    'scp': scp,
    'smv': smv,
    'get': get,
    'ssh': ssh,
    'del-all': delete_all,
    'del': delete,
    'ls': tele_ls,
    'csv': template_format,
    'bin': bin_exec,
    'enable-complete': enable_complete,
    'register-global': register_global_command,
    'gen-fig-script': gen_fig_script,
    'gen-zsh-comp': gen_zsh_comp,
}


def main():
    if len(sys.argv) < 2 or '-h' == sys.argv[1]:
        menu_output({'title': 'Qpro usage\n' if user_lang != 'zh' else 'Qpro 菜单\n',
                     'lines': [
                         ('init', 'let current dir be a Qpro project!' if user_lang != 'zh' else '使当前目录成为Qpro项目'),
                         ('-h', 'help' if user_lang != 'zh' else '帮助'),
                         ('create [bold magenta]<name>', 'create a Qpro project' if user_lang != 'zh' else '创建Qpro项目'),
                         ('update', 'update Qpro' if user_lang != 'zh' else '更新Qpro'),
                         ('ssh', 'login server by ssh' if user_lang != 'zh' else '通过SSH登录远程映射'),
                         (
                             'scp [bold magenta]<path>',
                             'upload path to default server target' if user_lang != 'zh' else '上传路径到默认的远程映射对应位置'
                         ),
                         (
                             'smv [bold magenta]<path>',
                             'delete after scp' if user_lang != 'zh' else '上传完成后删除文件或目录'
                         ),
                         (
                             'scp-init',
                             'upload all of project to server target' if user_lang != 'zh' else '上传当前全部内容到远程映射'
                         ),
                         (
                             'get [bold magenta]<path>',
                             'download file from server target' if user_lang != 'zh' else '从远程映射下载'
                         ),
                         (
                             'del [bold magenta]<path>',
                             'delete path in project' if user_lang != 'zh' else '同时删除本地及远程映射文件或目录'
                         ),
                         ('del-all', 'delete Qpro project' if user_lang != 'zh' else '销毁当前Qpro项目(本地+远程)'),
                         ('ls  [bold magenta]<path>', 'list element in path' if user_lang != 'zh' else '展示路径中的子项'),
                         ('enable-complete', 'enable complete' if user_lang != 'zh' else '启用Commander类的自动补全'),
                         ('register-global', 'register global command' if user_lang != 'zh' else '注册全局命令'),
                         ('gen-fig-script', 'generate fig autocomplete scripts' if user_lang != 'zh' else '生成Fig自动补全脚本'),
                         ('gen-zsh-comp', 'generate zsh autocomplete scripts' if user_lang != 'zh' else '生成zsh自动补全脚本'),
                         ('tmpm *', 'manage your template' if user_lang != 'zh' else '模板管理器'),
                         ('qrun *', 'run your Qpro project' if user_lang != 'zh' else '运行器'),
                         (
                             'detector -<p/f><p/f>',
                             'run beat detector for two source files' if user_lang != 'zh' else '对拍器'
                         )],
                     'prefix': 'Qpro'})
    elif 'update' == sys.argv[1]:
        os.system('pip3 install Qpro --upgrade')
    elif sys.argv[1] in func:
        try:
            func[sys.argv[1]]()
        except SystemExit:
            return
        except Exception:
            QproDefaultConsole.print_exception()
    elif sys.argv[1] == 'scp-init':
        scp_init(get_server_targets())
    elif 'init' != sys.argv[1]:
        QproDefaultConsole.print(
            QproErrorString, 'wrong usage! Run "Qpro -h" for help!' if user_lang != 'zh' else '请运行 "Qpro -h" 查看帮助!'
        )
    elif not os.path.exists(configure_name):
        pro_init()
    else:
        QproDefaultConsole.print(
            f"You have configured your project, see {project_configure_path} to adjust your configure!"
            if user_lang != 'zh' else
            f"你已经配置过这个项目啦, 查看配置表({project_configure_path})来调整它吧!"
        )


if __name__ == '__main__':
    main()



import re
import sys
import inspect
import argparse
from inspect import isfunction
from . import QproDefaultConsole, QproErrorString, user_lang


def str2bool(v):
    if isinstance(v, bool):
        return v
    if v.lower() in ('yes', 'true', 't', 'y', '1'):
        return True
    elif v.lower() in ('no', 'false', 'f', 'n', '0'):
        return False
    else:
        raise argparse.ArgumentTypeError('Boolean value expected.')


class Commander:
    def __init__(self, seg_flag: bool = False):
        """
        QuickProject的Commander类，帮助快速构建一个命令工具

        :param seg_flag: 是否将函数名中的'_'替换为'-'
        """
        self.command_table = {}
        self.fig_table = [{'name': '--help', 'description': '获取帮助'}]
        self.seg_flag = seg_flag

    def command(self):
        def wrapper(func):
            if not isfunction(func):
                raise TypeError(f'{func} not a function')
            param_doc = {i[0].strip(): i[1].strip() for i in re.findall(':param(.*?):(.*?)\n', func.__doc__, re.S)} if func.__doc__ else {}
            func_analyser = inspect.signature(func)
            func_args_parser = argparse.ArgumentParser()
            func_name = func.__name__.strip('_')
            description = func.__doc__.strip().split(':param')[0].strip() if func.__doc__ else func_name.replace('_', ' ')
            if self.seg_flag:
                func_name = func_name.replace('_', '-')
            func_fig = {'name': func_name, 'description': description, 'args': []}
            if func_name in self.command_table:
                raise Exception(f'{func} already in command table')
            for arg in func_analyser.parameters.values():
                _type = None
                _default = None
                if arg.annotation != arg.empty:
                    _type = arg.annotation
                if arg.default != arg.empty:
                    _default = arg.default
                if _default is None:
                    if _type == list:
                        func_args_parser.add_argument(f'-{arg.name}', type=str, nargs='+')
                        cur_args = {
                            'name': arg.name,
                            'description': param_doc.get(arg.name, f'<{arg.name}>'),
                            'isVariadic': True
                        }
                        if 'file' in arg.name.lower() or 'path' in arg.name.lower():
                            cur_args['template'] = ['filepaths', 'folders']
                        func_fig['args'].append({
                            'name': f'-{arg.name}',
                            'description': param_doc.get(arg.name, f'<{arg.name}>'),
                            'args': cur_args
                        })
                    else:
                        func_args_parser.add_argument(arg.name, type=_type if _type != bool else str2bool)
                        cur_args = {
                            'name': arg.name,
                            'description': param_doc.get(arg.name, f'<{arg.name}>'),
                        }
                        if 'file' in arg.name.lower() or 'path' in arg.name.lower():
                            cur_args['template'] = ['filepaths', 'folders']
                        func_fig['args'].append(cur_args)
                else:
                    if _type == list:
                        QproDefaultConsole.print(QproErrorString, f'{func_name}:', '"list" 类型不可以有默认值' if user_lang == 'zh' else '"list" type can not have default value')
                        return
                    _kw = {'required': False, 'type': _type, 'default': _default}
                    if _type == bool:
                        _kw.pop('type')
                        _kw['action'] = 'store_true'
                    func_args_parser.add_argument(f'--{arg.name}', **_kw)
                    cur_args = {
                        'name': arg.name,
                        'description': param_doc.get(arg.name, f'<{arg.name}>'),
                    }
                    if 'file' in arg.name.lower() or 'path' in arg.name.lower():
                        cur_args['template'] = ['filepaths', 'folders']
                    func_fig['args'].append({
                        'name': f'--{arg.name}',
                        'description': param_doc.get(arg.name, f'<{arg.name}>'),
                        'isOptional': True,
                        'args': cur_args
                    })
            self.fig_table.append(func_fig)
            self.command_table[func_name] = {'func': func, 'analyser': func_analyser, 'parser': func_args_parser, 'param_doc': param_doc, 'description': description}
        return wrapper

    def _check_args_(self, is_option: bool = True):
        res = False
        if is_option:
            for function in self.command_table:
                for arg in self.command_table[function]['analyser'].parameters.values():
                    _default = arg.default if arg.default != arg.empty else None
                    if _default is not None:
                        res = True
                        break
                if res:
                    break
        else:
            for function in self.command_table:
                for arg in self.command_table[function]['analyser'].parameters.values():
                    _default = arg.default if arg.default != arg.empty else None
                    if _default is None:
                        res = True
                        break
                if res:
                    break
        return res

    def help(self):
        from rich.table import Table
        from rich.box import SIMPLE_HEAVY

        has_requir = self._check_args_(False)
        has_option = self._check_args_(True)
        table = Table(show_edge=False, row_styles=['none', 'dim'], box=SIMPLE_HEAVY, title=f'[bold underline]帮助 HELP[/bold underline]\n')
        table.add_column('子命令\nSub Command', justify='center')
        table.add_column('描述\nDescription', justify='center')
        if has_requir:
            table.add_column('必填参数\nRequired Args', justify='center')
        if has_option:
            table.add_column('可选参数\nOptionnal Args', justify='center')
        for function in self.command_table:
            cur_line = ['[bold magenta]' + function + '[/bold magenta]']
            arg1, arg2 = [], []
            for arg in self.command_table[function]['analyser'].parameters.values():
                name = '[underline]' + arg.name + '[/underline]'
                _type = arg.annotation.__name__ if arg.annotation != arg.empty else 'Any'
                _default = arg.default if arg.default != arg.empty else None
                arg_str = f'{name}: [bold cyan]{_type}[/bold cyan]'
                if _default is not None:
                    if _type == 'str':
                        arg_str += f" = [dim]'[/dim][yellow]{_default}[/yellow][dim]'[/dim]"
                    elif _type == 'int' or _type == 'float':
                        arg_str += f' = [bold blue]{_default}[/bold blue]'
                    elif _type == 'bool':
                        arg_str += f' = [bold red]{_default}[/bold red]'
                    else:
                        arg_str += f' = {_default}'
                    arg2.append(arg_str)
                else:
                    arg1.append(arg_str)
            cur_line.append(self.command_table[function]['description'])
            if has_requir:
                cur_line.append(', '.join(arg1))
            if has_option:
                cur_line.append(', '.join(arg2))
            table.add_row(*cur_line)
        QproDefaultConsole.print(table, justify='center')

    def __command_complete__(self, route_path: list):
        """

        :param route_path:
        :return:
        """
        if not route_path:
            ls = [
                f"{i}:{self.command_table[i]['func'].__doc__.strip().split(':param')[0].strip().replace(' ', '_') if self.command_table[i]['func'].__doc__ else 'NONE'}"
                for i in self.command_table
            ]
            return '\n'.join(
                ls + ["--help:应用帮助" if user_lang == 'zh' else '--help:Application help']
            )
        call_func = route_path[0]
        has_args = [i.strip().strip('--') for i in route_path[1:]]
        if call_func not in self.command_table:
            return '错误:无该命令' if user_lang != 'zh' else 'ERROR:No such command'
        call_analyser = self.command_table[call_func]['analyser']
        param_doc = self.command_table[call_func]['param_doc']
        res = []
        for arg in call_analyser.parameters.values():
            if arg.name in has_args:
                continue
            if arg.default != arg.empty:
                res.append(f'--{arg.name}:{param_doc[arg.name].replace(" ", "_") if arg.name in param_doc else "No Description" if user_lang != "zh" else "无参数描述"}')
        return '\n'.join(res + ["--help:应用帮助" if user_lang == 'zh' else '--help:Application help'])

    def _fig_complete_(self):
        import json
        return json.dumps(self.fig_table, ensure_ascii=False)

    def __call__(self):
        if len(sys.argv) >= 2:
            if sys.argv[1] == '--qrun-commander-complete':
                if 'qrun' in sys.argv:
                    sys.argv.remove('qrun')
                return print(self.__command_complete__(sys.argv[2:]))
            elif sys.argv[1] == '--qrun-fig-complete':
                return print(self._fig_complete_())
            if sys.argv[1] == '--help':
                return self.help()
        try:
            func_name = sys.argv[1]
            sys.argv = sys.argv[:1] + sys.argv[2:]
            if func_name not in self.command_table:
                return QproDefaultConsole.print(QproErrorString, f'"{func_name}"' + (':无该命令' if user_lang != 'zh' else ':No such command'))
        except IndexError:
            return QproDefaultConsole.print(
                QproErrorString, '至少输入一个子命令!' if user_lang == 'zh' else 'Input at least one sub command!'
            )
        else:
            func_info = self.command_table[func_name]
            args = func_info['parser'].parse_args()
            try:
                return func_info['func'](**{i[0]: i[1] for i in args._get_kwargs()})
            except:
                return QproDefaultConsole.print_exception()

    def real_call(self, func_name: str, *args, **kwargs):
        """
        调用被装饰为命令的函数

        :param func_name: 注册的函数名
        :param args: 参数
        :param kwargs: 参数
        :return:
        """
        if self.seg_flag:
            func_name = func_name.replace('_', '-')
        if func_name not in self.command_table:
            return QproDefaultConsole.print(
                QproErrorString,  f'{func_name} 未被注册!' if user_lang == 'zh' else f'{func_name} not registered!'
            )
        return self.command_table[func_name]['func'](*args, **kwargs)



import pyperclip
from . import *

config = get_config(without_output='--qrun-commander-complete' in sys.argv or '--qrun-fig-complete' in sys.argv)
retain_arg = ['-br', '-b', '-r', '-h', '-i']
has_recog = {i: False for i in retain_arg}


def parseArgs():
    add_flag = True
    isPath = False
    programArgs, retainArgs = [], []
    conflictArgs = set()
    for i in sys.argv[1:]:
        if not add_flag:
            add_flag = True
            if isPath:
                programArgs.append(os.path.abspath(i))
            else:
                retainArgs.append(i)
            continue
        if i in retain_arg:
            if has_recog[i]:
                conflictArgs.add(i)
                programArgs.append(i)
            else:
                retainArgs.append(i)
                has_recog[i] = True
        elif i in ['-if', '-f', '--path']:
            add_flag = False
            isPath = i == '--path'
        else:
            programArgs.append(i)
    for i in conflictArgs:
        has_recog[i] = False
    return programArgs, retainArgs


argv, qrun_argv = parseArgs()


def run(use_txt=False, executable_file=str(config['executable_filename'])):
    if os.path.exists(executable_file):
        cmd = executable_file.replace(' ', '\ ') + ' '
    else:
        cmd = executable_file + ' '
    if '--qrun-commander-complete' in argv:
        if 'enable_complete' in config and config['enable_complete']:
            os.system(cmd + ' '.join(argv))
        else:
            print()
        return
    if '--qrun-fig-complete' in argv:
        if 'enable_complete' in config and config['enable_complete']:
            os.system(cmd + ' --qrun-fig-complete')
        else:
            print()
        return
    if argv:
        cmd += ' '.join(argv)
    if '--qrun-commander-complete' in argv and not config['enable_complete']:
        return print()
    if cmd.strip():
        cmd += (' < "' + config['input_file'] + '"' if use_txt else '')
        os.system(cmd)


def main():
    to_build = '-b' in qrun_argv or '-br' in qrun_argv
    to_run = '-r' in qrun_argv or '-b' not in qrun_argv
    filename = rt_dir + config['compile_filename'] if config['compile_filename'] else ''
    flag = False
    if has_recog['-h']:
        menu_output({'title': 'qrun usage\n',
                     'lines': [
                            ('-b', 'build' if user_lang != 'zh' else '编译'),
                            ('qrun [bold green][-r]', 'run' if user_lang != 'zh' else '运行'),
                            ('-br', 'build and run' if user_lang != 'zh' else '编译且运行'),
                            ('-h', 'help' if user_lang != 'zh' else '帮助'),
                            ('-i', 'use input.txt as input' if user_lang != 'zh' else '使用默认输入文件作为输入'),
                            ('-if [bold magenta]<file>', 'set file as input' if user_lang != 'zh' else '输入重定向'),
                            ('-if [bold magenta]-paste', 'use Clipboard content as input' if user_lang != 'zh' else '输入重定向到粘贴板'),
                            ('-f  [bold magenta]<file>', 'set file as build file' if user_lang != 'zh' else '指定源文件'),
                            ('*', 'add parameters for program' if user_lang != 'zh' else '程序的任何其他参数')],
                     'prefix': 'qrun '})
        if '-h' not in argv:
            return
    if '-f' in qrun_argv:
        index = qrun_argv.index('-f')
        if index == len(qrun_argv) - 1:
            return QproDefaultConsole.print(QproErrorString, 'No file with -f' if user_lang != 'zh' else '-f后没有指定文件')
        filename = qrun_argv[index + 1] if qrun_argv[index + 1] != '__ignore__' else config['compile_filename']
        if not os.path.exists(filename):
            return QproDefaultConsole.print(
                QproErrorString, (
                    'No such file: "{filename}"' if user_lang != 'zh' else '没有这个文件: "{filename}"'
                ).format(filename=filename)
            )
        filename = os.path.abspath(filename)
        flag = filename != config['compile_filename']
    if '-if' in qrun_argv:
        index = qrun_argv.index('-if')
        if index == len(qrun_argv) - 1:
            QproDefaultConsole.print(QproErrorString, 'No file with -if' if user_lang != 'zh' else '-if后没有指定文件')
        tmp_file = qrun_argv[index + 1]
        if tmp_file == '-paste':
            with open(rt_dir + config['input_file'], 'w') as file:
                file.write(pyperclip.paste())
        elif tmp_file == '__ignore__':
            pass
        else:
            __input_file__ = tmp_file
            if not os.path.exists(__input_file__):
                return QproDefaultConsole.print(
                    QproErrorString, (
                        'No such file: "{input_file}"' if user_lang != 'zh' else '没有这个文件: "{input_file}"'
                    ).format(input_file=__input_file__)
                )
            config['input_file'] = os.path.abspath(__input_file__)
    o_file = config['executable_filename']
    record_file_name = os.path.basename(filename).split('.')[0]

    os.chdir(rt_dir)

    if config['compile_tool'] and to_build:
        cmd = config['compile_tool']
        if filename and config['compile_filename']:
            cmd = cmd.replace(config['compile_filename'], filename)
        os.system(cmd)
    if to_run:
        run('-i' in qrun_argv or '-if' in qrun_argv, o_file)
    if config['compile_tool'] and flag:
        if config['compile_tool'].split()[0] == 'javac':
            os.remove('dist' + dir_char + record_file_name + '.class')
        else:
            os.remove(o_file)


if __name__ == '__main__':
    main()



import sys
import os
from QuickProject import *

config = get_config()
is_cpp = config['compile_filename'].endswith('cpp')
algorithm_name = 'main'
temp_name = 'main'


def match_algorithm():
    with open(config['compile_filename'], 'r') as file:
        import re
        try:
            content = re.findall('__TMPM_START__(.*?)__TMPM_END__', file.read(), re.S)[0].strip()
        except IndexError:
            return QproDefaultConsole.print(
                QproErrorString,
                'No template index found! Insert "__TMPM_START__" and "__TMPM_END__" to your code!'
                if user_lang != 'zh' else
                '没有找到模板桩! 请插入"__TMPM_START__" 和 "__TMPM_END__"到默认源文件中!'
            )
    content = content.replace('__TMPM__', '')
    return content


def write_algorithm(temp, algorithm, content, mode):
    with open(config['template_root'] + temp, mode) as file:
        file.write('\n## %s\n\n```%s\n' % (algorithm, 'c++' if is_cpp else 'c'))
        file.write(content)
        file.write('\n```\n')


def create():
    global temp_name, algorithm_name
    indx = sys.argv.index('-c')
    try:
        temp_name = sys.argv[indx + 1]
        algorithm_name = sys.argv[indx + 2]
    except IndexError:
        if temp_name != 'main':
            with open(config['compile_filename'], 'r') as file:
                ct = file.read()
            with open(config['template_root'] + temp_name, 'w') as file:
                file.write(ct)
            return
        else:
            return QproDefaultConsole.print(
                QproWarnString,
                'usage: tmpm -c <template> [algorithm]'
                if user_lang != 'zh' else
                '使用: tmpm -c <模板> [算法]'
            )
    temp_name += '.md'
    if os.path.exists(config['template_root'] + temp_name):
        from rich.prompt import Prompt
        if Prompt.ask((
            'Template {temp_name} is already exist, would you cover it?[y/n]'
            if user_lang != 'zh' else
            '模板 {temp_name} 已经存在, 是否覆盖它?[y/n]'
        ).format(temp_name=temp_name), default='n') == 'n':
            exit(0)
    content = match_algorithm()
    write_algorithm(temp_name, algorithm_name, content, 'w')


def append():
    global temp_name, algorithm_name
    indx = sys.argv.index('-a')
    try:
        temp_name = sys.argv[indx + 1] + '.md'
        algorithm_name = sys.argv[indx + 2]
    except IndexError:
        return QproDefaultConsole.print(
            QproWarnString,
            'usage: tmpm -a <template> [algorithm]'
            if user_lang != 'zh' else
            '使用: tmpm -a <模板> [算法]'
        )
    if os.path.exists(config['template_root'] + temp_name):
        content = match_algorithm()
        write_algorithm(temp_name, algorithm_name, content, 'a')
    else:
        sys.argv[indx] = '-c'
        create()


def join():
    global temp_name
    try:
        temp_name = sys.argv[1] + '.md'
    except IndexError:
        exit('usage: tmpm template')
    if os.path.exists(config['template_root'] + temp_name):
        with open(config['template_root'] + temp_name, 'r') as file:
            import re
            from rich.prompt import Prompt

            content = re.findall('##(.*?)\n.*?```.*?\n(.*?)```', file.read(), re.S)
            for i, v in enumerate(content):
                QproDefaultConsole.print('[%d] %s' % (i + 1, v[0].strip()), end=' ' if i + 1 % 10 else '\n')
            indx = int(Prompt.ask(('%sChoose' if user_lang != 'zh' else '%s选择') % ('\n' if len(content) % 10 else ''))) - 1
            content = content[indx]
        with open(config['compile_filename'], 'r') as file:
            content = file.read().replace('__TMPM__', content[1].strip())
        with open(config['compile_filename'], 'w') as file:
            file.write(content)
    else:
        return QproDefaultConsole.print(
            QproErrorString, (
                'No template named: {temp_name}'
                if user_lang != 'zh' else
                '没有模板: {temp_name}'
            ).format(temp_name=temp_name)
        )


def h():
    menu_output({'title': 'tmpm usage\n' if user_lang != 'zh' else '模板管理器菜单\n',
                 'lines': [
                     ('-h', 'for help' if user_lang != 'zh' else '帮助'),
                     ('-i', 'init content as template/main' if user_lang != 'zh' else '恢复源文件内容为template/main'),
                     ('-r', 'select copy and init' if user_lang != 'zh' else '选择备份然后恢复'),
                     (
                         '-r [bold magenta]<backup>',
                         'revert "compile_filename" from template/<backup>'
                         if user_lang != 'zh' else '恢复源文件内容为template/<backup>'
                     ),
                     (
                         '-c [bold magenta]<backup>',
                         'create or cover a <backup> of "compile_filename"' if user_lang != 'zh' else '创建或覆盖备份'
                     ),
                     (
                         '-c [bold magenta]<template> [bold magenta]<algorithm>',
                         'create template and write algorithm' if user_lang != 'zh' else '创建模板然后写入算法'
                     ),
                     (
                         '-a [bold magenta]<template> [bold magenta]<algorithm>',
                         'add algorithm to template' if user_lang != 'zh' else '添加算法到模板'
                     ),
                     (
                         'tmpm    [bold magenta]<template>',
                         'insert algorithm in <template>' if user_lang != 'zh' else '在模板桩插入模板库中的算法'
                     )],
                 'prefix': 'tmpm'})


def init(file_name: str = 'main'):
    with open(str(config['template_root']) + file_name, 'r') as file:
        content = file.read()
    with open(config['compile_filename'], 'w') as file:
        file.write(content)


def revert():
    indx = sys.argv.index('-r')
    try:
        file_name = sys.argv[indx + 1]
    except IndexError:
        from rich.prompt import Prompt

        ls = os.listdir(config['template_root'])
        rls = []
        cnt = 1
        stdSuffix = config['compile_filename'].strip().split('.')[-1]
        for i in ls:
            iSuffix = i.split('.')[-1]
            if iSuffix != stdSuffix and '.' in i:
                continue
            QproDefaultConsole.print('[%d] %s' % (cnt, i), end='\t' if cnt % 8 else '\n')
            rls.append(i)
            cnt += 1
        if cnt % 8:
            QproDefaultConsole.print()
        try:
            indx = int(Prompt.ask('选择' if user_lang == 'zh' else 'Choose'))
            if indx < 0 or indx > len(rls):
                raise IndexError
        except:
            return QproDefaultConsole.print(QproErrorString, 'Choose out of index' if user_lang == 'zh' else '选择')
        file_name = rls[indx - 1]
        return init(file_name)
    if os.path.exists(config['template_root'] + file_name):
        init(file_name)
    else:
        return QproDefaultConsole.print(QproErrorString, 'No such backup' if user_lang != 'zh' else '没有这个备份')


def complete():
    if rt_dir == os.path.abspath('.') + dir_char and not os.path.exists(project_configure_path):
        return print()
    print(' '.join(os.listdir(rt_dir + config['template_root'])))


def main():
    os.chdir(rt_dir)

    if len(sys.argv) == 1:
        h()
    elif '-h' in sys.argv:
        h()
    elif '-c' in sys.argv:
        create()
    elif '-a' in sys.argv:
        append()
    elif '-r' in sys.argv:
        revert()
    elif '-i' in sys.argv:
        init()
    elif '-complete' in sys.argv:
        complete()
    else:
        join()


if __name__ == '__main__':
    main()



zsh_comp_template = """#compdef __proj_name__

local cur prev
cur=${words[CURRENT]}
prev=${words[CURRENT-1]}

(( $+functions[___proj_name___args_comp] )) || 
___proj_name___args_comp() {
    local -a opt_args
    __sub_commands_args__
    _describe subcommand opt_args
}

(( $+functions[___proj_name___main] )) || 
___proj_name___main() {
    local -a args
    args=(
        __sub_commands__
    )

    _describe -t common-commands 'common commands' args && _ret=0
}

if (( ${#words} >= 3 )); then
    ___proj_name___args_comp
else
    ___proj_name___main
fi
"""



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()
VERSION = '0.9.13'

setup(
    name='Qpro',
    version=VERSION,
    description='Small but powerful command line IDE.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    keywords='script for CLionProjects',
    author='RhythmLian',
    url="https://github.com/Rhythmicc/QuickProject",
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=True,
    install_requires=['pyperclip', 'rich', 'gitpython', 'inquirer-rhy'],
    entry_points={
        'console_scripts': [
            'Qpro = QuickProject.Qpro:main',
            'qrun = QuickProject.qrun:main',
            'tmpm = QuickProject.tmpm:main',
            'detector = QuickProject.detector:main'
        ]
    },
)



from inquirer_rhy.prompt import prompt
import sys
from rich.console import Console

QproDefaultConsole = Console()
system = sys.platform
if system.startswith('win'):
    dir_char = '\\'
else:
    dir_char = '/'

default_language = {
    'type': 'input',
    'name': 'default_language',
    'message': """Select your language, the flowing content is available choice
  选择你的语言，下述内容为合法选项

    zh  (Chinese) en  (English)  jp  (Japanese) kor (Korean)   fra (French), 
    spa (Spanish) th  (Thailand) ara (al-ummah) ru  (Russian)  pt  (Portuguese), 
    de  (Germany) it  (Italy)    el  (Greece)   nl  (Poland)   bul (Bulgaria),
    est (Estonia) dan (Denmark)  fin (Finland)  cs  (Czech)    rom (Romania),
    slo (Iceland) swe (Sweden)   hu  (Hungary)  vie (Vietnam)

  Input the default language | 输入默认语言:""",
    'validate': lambda val: val in ['zh', 'en', 'jp', 'kor', 'fra',
                                    'spa', 'th', 'ara', 'ru', 'pt',
                                    'de', 'it', 'el', 'nl', 'bul',
                                    'est', 'dan', 'fin', 'cs', 'rom',
                                    'slo', 'swe', 'hu', 'vie', 'Not Set | 暂不配置'],
    'default': 'Not Set | 暂不配置'
}

default_pip = {
    'type': 'input',
    'name': 'default_pip',
    'message': 'Input the default pip | 输入默认pip:',
    'default': 'pip3'
}


class QproConfig:
    import json

    def __init__(self, configPath, isExists):
        self.path = configPath
        if isExists:
            try:
                with open(configPath, 'r') as f:
                    self.config = QproConfig.json.loads(f.read())
            except:
                with open(configPath, 'r', encoding='utf8') as f:
                    self.config = QproConfig.json.loads(f.read(), encoding='utf8')
        else:
            self.config = QproConfig.json.loads("""{
    "default_language": "",
    "default_pip": ""
}""")
            res = prompt([default_language, default_pip])
            self.config['default_language'] = res['default_language'] if res['default_language'] != 'Not Set | 暂不配置' else 'en'
            self.config['default_pip'] = res['default_pip']
            self.update()
            QproDefaultConsole.print(
                '\nYour configuration table has been stored\n你的配置表被存储在: [bold green]' + '%s' % configPath + '[/bold '
                                                                                                            'green]')
            QproDefaultConsole.print('[bold red]\nqs will not use your configuration do '
                                     'anything!\nQpro不会用您的配置表做任何事情![/bold red]')
            prompt({
                'type': 'confirm',
                'message': 'Confirm | 确认',
                'name': 'done',
                'default': True
            })

    def update(self):
        with open(self.path, 'w') as f:
            QproConfig.json.dump(self.config, f, indent=4, separators=(',', ': '))

    def select(self, key: str):
        if key not in self.config:
            exec(f'res = prompt([{key}])')
            exec(f"self.config['{key}'] = res['{key}']")
            self.update()
        return self.config[key]


def _ask(question):
    try:
        return prompt(question)[question['name']]
    except:
        exit(0)



default_Qpro_zh_content_template = \
    """const completionSpec: Fig.Spec = {
  'name': 'Qpro',
  'description': 'Quick Project',
  'options': [
    {
      'name': '-h',
      'description': '帮助'
    },
  ],
  'subcommands': [{
    'name': 'init',
    'description': '使当前目录成为Qpro项目',
  }, {
    'name': 'create',
    'description': '创建Qpro项目',
    'args': {'name': 'name', 'description': '项目名'}
  }, {
    'name': 'update',
    'description': '更新Qpro'
  }, {
    'name': 'adjust',
    'description': '调整配置表'
  }, {
    'name': 'ssh',
    'description': '通过SSH登录远程映射'
  }, {
    'name': 'scp',
    'description': '上传路径到默认的远程映射对应位置',
    'args': {'name': 'path', 'description': '路径', 'template': ["filepaths", "folders"]}
  }, {
    'name': 'smv',
    'description': '移动路径到默认的远程映射对应位置',
    'args': {'name': 'path', 'description': '路径', 'template': ["filepaths", "folders"]}
  }, {
    'name': 'scp-init',
    'description': '上传当前全部内容到远程映射'
  }, {
    'name': 'get',
    'description': '从远程映射下载',
    'args': {'name': 'path', 'description': '路径', 'template': ["filepaths", "folders"]}
  }, {
    'name': 'del',
    'description': '同时删除本地及远程映射文件或目录',
    'args': {'name': 'path', 'description': '路径', 'template': ["filepaths", "folders"]}
  }, {
    'name': 'del-all',
    'description': '同时删除本地及远程映射文件或目录'
  }, {
    'name': 'ls',
    'description': '展示路径中的子项',
    'args': {'name': 'path', 'description': '路径', 'template': ["filepaths", "folders"]}
  }, {
    'name': 'enable-complete',
    'description': '支持Commander应用的自动补全'
  }, {
    'name': 'register-global',
    'description': '注册全局命令',
    'args': {'name': '--disable_global_command', 'description': '禁用全局命令', 'template': ["boolean"]}
  }, {
    'name': 'gen-fig-script',
    'description': '生成Fig自动补全脚本并重新编译'
  }, {
    'name': 'gen-zsh-comp',
    'description': '生成zsh自动补全脚本'
  }, {
    'name': 'csv',
    'description': '将旧的Qpro配置表改为json格式'
  }, {
    'name': 'bin',
    'description': '执行Qpro自定义全局命令',
    'subcommands': --Qpro-bin-subcommands-gen--
    }
  ],
};
export default completionSpec;
"""

default_Qpro_en_content_template = \
    """const completionSpec: Fig.Spec = {
  'name': 'Qpro',
  'description': 'Quick Project',
  'options': [
    {
      'name': '-h',
      'description': 'help'
    },
  ],
  'subcommands': [{
    'name': 'init',
    'description': 'Make the current path a Qpro project',
  }, {
    'name': 'create',
    'description': 'Create a Qpro project',
    'args': {'name': 'name', 'description': 'project name'}
  }, {
    'name': 'update',
    'description': 'Update Qpro'
  }, {
    'name': 'adjust',
    'description': 'Adjust configuration'
  }, {
    'name': 'ssh',
    'description': 'Login through SSH'
  }, {
    'name': 'scp',
    'description': 'Upload path to the default remote mapping location',
    'args': {'name': 'path', 'description': 'path', 'template': ["filepaths", "folders"]}
  }, {
    'name': 'smv',
    'description': 'Move path to the default remote mapping location',
    'args': {'name': 'path', 'description': 'path', 'template': ["filepaths", "folders"]}
  }, {
    'name': 'scp-init',
    'description': 'Upload all files to the remote mapping'
  }, {
    'name': 'get',
    'description': 'Download from remote mapping',
    'args': {'name': 'path', 'description': 'path', 'template': ["filepaths", "folders"]}
  }, {
    'name': 'del',
    'description': 'Delete local and remote mapping file or directory',
    'args': {'name': 'path', 'description': 'path', 'template': ["filepaths", "folders"]}
  }, {
    'name': 'del-all',
    'description': 'Delete all local and remote mapping files or directories'
  }, {
    'name': 'ls',
    'description': 'List the subitems of the path',
    'args': {'name': 'path', 'description': 'path', 'template': ["filepaths", "folders"]}
  }, {
    'name': 'enable-complete',
    'description': 'Support the auto completion of the Commander application'
  }, {
    'name': 'register-global',
    'description': 'Register global command'
  }, {
    'name': 'gen-fig-script',
    'description': 'Generate Fig autocompletion script and recompile'
  }, {
    'name': 'csv',
    'description': 'Convert old Qpro configuration to json format'
  }, {
    'name': 'bin',
    'description': 'Execute Qpro custom global command',
    'subcommands': --Qpro-bin-subcommands-gen--
    }
  ],
};
export default completionSpec;
"""

default_qrun_zh_content_template = \
    """const completionSpec: Fig.Spec = {
  name: "qrun",
  description: "QuickProject的运行器",
  args: { 
    name: 'command',
    isCommand: true,
    generators: {
      script: 'qrun --qrun-fig-complete',
      postProcess: function (out) {
        if (out) return JSON.parse(out);
        return [];
      }
    },
    isOptional: true
  },
  options: [{
    name: "-b",
    description: "编译",
  }, {
    name: "-br",
    description: "编译且运行",
  }, {
    name: "-r",
    description: "运行",
  }, {
    name: "-f",
    description: "指定源文件",
    args: {name: 'source', description: '源文件', template: 'filepaths'}
  }, {
    name: "-h",
    description: "运行器帮助",
  }, {
    name: "-i",
    description: "使用默认输入文件作为输入",
  }, {
    name: "-if",
    description: "设置输入文件",
    args: {name: 'input', description: '输入文件', template: 'filepaths'}
  }]
};
export default completionSpec;    
"""

default_qrun_en_content_template = \
    """const completionSpec: Fig.Spec = {
  name: "qrun",
  description: "QuickProject runner",
  args: { 
    name: 'command',
    isCommand: true,
    generators: {
      script: 'qrun --qrun-fig-complete',
      postProcess: function (out) {
        if (out) return JSON.parse(out);
        return [];
      }
    },
    isOptional: true
  },
  options: [{
    name: "-b",
    description: "Compile",
  }, {
    name: "-br",
    description: "Compile and run",
  }, {
    name: "-r",
    description: "Run",
  }, {
    name: "-f",
    description: "Set source file",
    args: {name: 'source', description: 'source file', template: 'filepaths'}
  }, {
    name: "-h",
    description: "Runner help",
  }, {
    name: "-i",
    description: "Use default input file as input",
  }, {
    name: "-if",
    description: "Set input file",
    args: {name: 'input', description: 'input file', template: 'filepaths'}
  }]
};
export default completionSpec;    
"""

default_detector_zh_content_template = \
    """const completionSpec: Fig.Spec = {
  name: "detector",
  description: "Qpro的对拍器",
  options: [{
    name: '-pp',
    description: '程序-程序对拍',
  }, {
    name: '-pf',
    description: '程序-文件对拍',
  }, {
    name: '-fp',
    description: '文件-程序对拍',
  }, {
    name: '-ff',
    description: '文件-文件对拍',
  }]
};
export default completionSpec;
"""

default_detector_en_content_template = \
    """const completionSpec: Fig.Spec = {
  name: "detector",
  description: "Qpro detector",
  options: [{
    name: '-pp',
    description: 'program-program detector',
  }, {
    name: '-pf',
    description: 'program-file detector',
  }, {
    name: '-fp',
    description: 'file-program detector',
  }, {
    name: '-ff',
    description: 'file-file detector',
  }]
};
export default completionSpec;
"""


default_tmpm_zh_content_template = \
    """const completionSpec: Fig.Spec = {
  name: "tmpm",
  description: "Qpro template manager",
  options: [{
    name: "-h",
    description: "帮助",
  }, {
    name: "-i",
    description: "恢复源文件内容为template/main"
  }, {
    name: "-r",
    description: '选择备份然后恢复',
    args: {name: 'backup', description: '备份名', isOptional: true}
  }, {
    name: "-c",
    description: '创建或覆盖备份和模板',
    args: [
      {name: 'backup', description: '备份名 | 模板名'},
      {name: 'algorithm', description: '代码片段名(创建备份时勿设置此参数)', isOptional: true}
    ]
  }, {
    name: "-a",
    description: '添加算法到模板',
    args: [
      {name: 'template', description: '模板名'},
      {name: 'algorithm', description: '代码片段名'}
    ]
  }]
};
export default completionSpec;
"""

default_tmpm_en_content_template = \
    """const completionSpec: Fig.Spec = {
  name: "tmpm",
  description: "Qpro template manager",
  options: [{
    name: "-h",
    description: "Help",
  }, {
    name: "-i",
    description: "Restore source file content to template/main"
  }, {
    name: "-r",
    description: 'Select backup and restore',
    args: {name: 'backup', description: 'backup name', isOptional: true}
  }, {
    name: "-c",
    description: 'Create or cover backup and template',
    args: [
      {name: 'backup', description: 'backup name | template name'},
      {name: 'algorithm', description: 'algorithm (ignore this arg if creating backup)', isOptional: true}
    ]
  }, {
    name: "-a",
    description: 'Add algorithm to template',
    args: [
      {name: 'template', description: 'template name'},
      {name: 'algorithm', description: 'algorithm name'}
    ]
  }]
};
export default completionSpec;
"""

default_custom_command_template = \
    """const completionSpec: Fig.Spec = __CUSTOM_COMMAND_SPEC__;
export default completionSpec;
"""



import difflib
import os
import shutil
import webbrowser as wb
import sys
import tkinter as tk
from tkinter import filedialog, messagebox
from QuickProject import get_config


def read_file(filename):
    try:
        with open(filename.replace('\ ', ' '), 'r') as f:
            return f.readlines()
    except IOError:
        messagebox.showerror("ERROR:", "没有找到文件:%s" % filename)
        return


def compare_file(f1, f2, out_file):
    file1_content = read_file(f1.strip())
    file2_content = read_file(f2.strip())
    d = difflib.HtmlDiff()
    result = d.make_file(file1_content, file2_content)
    with open(out_file, 'w') as f:
        f.writelines(result)


def get_path1():
    global file1
    file1 = filedialog.askopenfilename()
    path1.set(file1)


def get_path2():
    global file2
    file2 = filedialog.askopenfilename()
    path2.set(file2)


def get_path3():
    global file3
    file3 = filedialog.askopenfilename()
    path3.set(file3)
    ll.configure(fg='black')


def remove(path):
    if os.path.exists(path):
        if os.path.isdir(path):
            shutil.rmtree(path)
        else:
            os.remove(path)


def cmp():
    if not file1 or not file2:
        messagebox.showerror("文件数不足", "请选择两个文件进行对比")
        return
    cp1 = len(sys.argv) < 2 or sys.argv[1].startswith('-p')
    cp2 = len(sys.argv) < 2 or sys.argv[1].endswith('p')
    file_path1 = 'content1'
    file_path2 = 'content2'
    if file3 == '使用默认输入文件':
        if cp1:
            os.system('qrun -br -i -f %s > content1' % file1.replace(' ', '\ '))
        else:
            file_path1 = file1
        if cp2:
            os.system('qrun -br -i -f %s > content2' % file2.replace(' ', '\ '))
        else:
            file_path2 = file2
    else:
        if cp1:
            os.system('qrun -br -if %s -f %s > content1' % (file3.replace(' ', '\ '), file1.replace(' ', '\ ')))
        else:
            file_path1 = file1
        if cp2:
            os.system('qrun -br -if %s -f %s > content2' % (file3.replace(' ', '\ '), file2.replace(' ', '\ ')))
        else:
            file_path2 = file2
    compare_file(file_path1, file_path2, './res.html')
    res_path = os.path.abspath('./res.html')
    wb.open('file://%s' % res_path)
    remove('content1')
    remove('content2')


status = {
    '-pp': '程序--程序',
    '-pf': '程序--文件',
    '-fp': '文件--程序',
    '-ff': '文件--文件',
}

title_status = '默认' if len(sys.argv) < 2 else status[sys.argv[1]]
width = 30
win = tk.Tk()
win.title('对拍器 %s' % title_status)
file1, path1 = '', tk.StringVar()
file2, path2 = '', tk.StringVar()
file3, path3 = '使用默认输入文件', tk.StringVar()
path3.set(file3)
ll = tk.Label(win, textvariable=path3, width=width, fg='red')


def main():
    if not get_config() and sys.argv[1] != '-ff':
        from QuickProject import QproDefaultConsole, QproErrorString
        return QproDefaultConsole.print(QproErrorString, "You must run \"Qpro -init\" first")
    tk.Label(win, text='%12s' % "文件1路径:").grid(row=0, column=0)
    tk.Label(win, textvariable=path1, width=width).grid(row=0, column=1)
    tk.Button(win, text="路径选择", command=get_path1).grid(row=0, column=2)
    tk.Label(win, text='%12s' % "文件2路径:").grid(row=1, column=0)
    tk.Label(win, textvariable=path2, width=width).grid(row=1, column=1)
    tk.Button(win, text="路径选择", command=get_path2).grid(row=1, column=2)
    if len(sys.argv) < 2 or 'p' in sys.argv[1]:
        tk.Label(win, text='%12s' % "输入文件:").grid(row=2, column=0)
        ll.grid(row=2, column=1)
        tk.Button(win, text="路径选择", command=get_path3).grid(row=2, column=2)
    tk.Button(win, text="对比", command=cmp).grid(row=3, column=1)
    tk.mainloop()


if __name__ == '__main__':
    main()
